/* GStreamer H264 video encoder
 * Copyright (C) 2010 Miguel Verdu <miguel.verdu@nokia.com>
 *
 * h264encoder.h: Common interface for H264 vidoe encoder implementations
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __MEEGO_GST_H264_ENCODER_H__
#define __MEEGO_GST_H264_ENCODER_H__

#ifndef GST_USE_UNSTABLE_API
#warning "MeegoGstH264Encoder is unstable API and may change in future."
#warning "You can define GST_USE_UNSTABLE_API to avoid this warning."
#endif

#include <gst/gst.h>
#include <gst/interfaces/meegointerfaces-enumtypes.h>

G_BEGIN_DECLS

#define MEEGO_GST_TYPE_H264_ENCODER \
  (meego_gst_h264_encoder_get_type ())
#define MEEGO_GST_H264_ENCODER(obj) \
  (GST_IMPLEMENTS_INTERFACE_CHECK_INSTANCE_CAST ((obj), MEEGO_GST_TYPE_H264_ENCODER, \
                                                 MeegoGstH264Encoder))
#define MEEGO_GST_H264_ENCODER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), MEEGO_GST_TYPE_H264_ENCODER, MeegoGstH264EncoderClass))
#define MEEGO_GST_IS_H264_ENCODER(obj) \
  (GST_IMPLEMENTS_INTERFACE_CHECK_INSTANCE_TYPE ((obj), MEEGO_GST_TYPE_H264_ENCODER))
#define MEEGO_GST_IS_H264_ENCODER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), MEEGO_GST_TYPE_H264_ENCODER))
#define MEEGO_GST_H264_ENCODER_GET_CLASS(inst) \
  (G_TYPE_INSTANCE_GET_INTERFACE ((inst), MEEGO_GST_TYPE_H264_ENCODER, MeegoGstH264EncoderClass))
  
 

/* Interface property names */
#define MEEGO_GST_H264_ENCODER_PROP_PROFILE 				"profile"

#define MEEGO_GST_H264_ENCODER_PROP_RATE_CONTROL_MODE 		"rate-control-mode"
#define MEEGO_GST_H264_ENCODER_PROP_BITRATE	 				"bit-rate"
#define MEEGO_GST_H264_ENCODER_PROP_MAX_BITRATE 			"max-bit-rate"

// NONE
#define MEEGO_GST_H264_ENCODER_PROP_MIN_QP 					"min-qp"
#define MEEGO_GST_H264_ENCODER_PROP_MAX_QP 					"max-qp"
#define MEEGO_GST_H264_ENCODER_PROP_INTRA_QP 				"intra-qp"
#define MEEGO_GST_H264_ENCODER_PROP_INTER_QP 				"inter-qp"
// CBR
#define MEEGO_GST_H264_ENCODER_PROP_DELAY		 			"delay"
// VBR
#define MEEGO_GST_H264_ENCODER_PROP_RATE_TOLERANCE			"rate-tol"

#define MEEGO_GST_H264_ENCODER_PROP_KEY_FRAME_PERIOD 	"keyframe-period"
#define MEEGO_GST_H264_ENCODER_PROP_SLICE_SIZE_MB 		"slice-size-mb"
#define MEEGO_GST_H264_ENCODER_PROP_INTRA_REFRESH_MODE	"intra-refresh-mode"
#define MEEGO_GST_H264_ENCODER_PROP_INTRA_REFRESH_RATE	"intra-refresh-rate"



/**
 * GstMeegoH264EncoderProfile:
 * @MEEGO_GST_H264_ENCODER_PROFILE_NONE: No profile specified, no restrictions
 * @MEEGO_GST_H264_ENCODER_PROFILE_BASELINE: Baseline profile
 * @MEEGO_GST_H264_ENCODER_PROFILE_MAIN: Main Profile
 * @MEEGO_GST_H264_ENCODER_PROFILE_HIGH: High Profile
 *
 * Specifies the profile the encoder will use. It restricts the set of tools
 * used in the encoding process as defined in the standard.
 *
 **/
typedef enum
{
  MEEGO_GST_H264_ENCODER_PROFILE_NONE = 0,
  MEEGO_GST_H264_ENCODER_PROFILE_BASELINE,
  MEEGO_GST_H264_ENCODER_PROFILE_MAIN,
  MEEGO_GST_H264_ENCODER_PROFILE_HIGH
} GstMeegoH264EncoderProfile;

/**
 * GstMeegoH264EncoderIntraRefreshMode:
 * @MEEGO_GST_H264_ENCODER_INTRA_REFRESH_NONE: No intra refresh specified. Let 
 * the encoder use its normal decision logic based on rate control constrains. Basically random
 * @MEEGO_GST_H264_ENCODER_INTRA_REFRESH_ADAPTATIVE: Adaptative Intra refresh. The number
 * of I MB's changes depending on other factors (motion, errors etc...)
 * @MEEGO_GST_H264_ENCODER_INTRA_REFRESH_CYCLIC: Cyclic Intra refresh. Ensures that the
 * frame is completely updated after #MeegoGstH264Encoder:intra-refresh-rate seconds.
 * @MEEGO_GST_H264_ENCODER_INTRA_REFRESH_OTHER: Open door to other algorithms
 *
 * List of possible modes of Intra refresh used by the encoder
 * Intra refresh controls the how many and where I-MacroBlocks are inserted
 * in the middle of a P frame. 
 *
 **/
typedef enum
{
  MEEGO_GST_H264_ENCODER_INTRA_REFRESH_NONE = 0,
  MEEGO_GST_H264_ENCODER_INTRA_REFRESH_ADAPTATIVE,
  MEEGO_GST_H264_ENCODER_INTRA_REFRESH_CYCLIC,
  MEEGO_GST_H264_ENCODER_INTRA_REFRESH_OTHER
} GstMeegoH264EncoderIntraRefreshMode;

/**
 * GstMeegoH264EncoderRateControlMode:
 * @MEEGO_GST_H264_ENCODER_RATE_CONTROL_NONE: Bitrate controled by QP paramters
 * @MEEGO_GST_H264_ENCODER_RATE_CONTROL_VARIABLE: Variable Bitrate. The target bitrate
 * is allowed to deviate by #MeegoGstH264Encoder:rate-tol %
 * @MEEGO_GST_H264_ENCODER_RATE_CONTROL_CONSTANT: Constant Bitrate. Target bitrate
 * is must be kept. Deviations are controled by the #MeegoGstH264Encoder:delay propertie
 * @MEEGO_GST_H264_ENCODER_RATE_CONTROL_OTHER: Other
 *
 * List of possible bitrate control methods the encoder may use.
 * 
 *
 **/
typedef enum
{
  MEEGO_GST_H264_ENCODER_RATE_CONTROL_NONE = 0,
  MEEGO_GST_H264_ENCODER_RATE_CONTROL_VARIABLE,
  MEEGO_GST_H264_ENCODER_RATE_CONTROL_CONSTANT,
  MEEGO_GST_H264_ENCODER_RATE_CONTROL_OTHER
} GstMeegoH264EncoderRateControlMode;

/**
 * MeegoGstH264Encoder:
 *
 * Opaque #MeegoGstH264Encoder data structure.
 */
typedef struct _MeegoGstH264Encoder MeegoGstH264Encoder;
typedef struct _MeegoGstH264EncoderClass MeegoGstH264EncoderClass;

/**
 * MeegoGstH264EncoderClass:
 * @klass: parent interface type.
 * #MeegoGstH264Encoder interface
 */

struct _MeegoGstH264EncoderClass {
  GTypeInterface klass;

   /* interface methods */
  
    /*< private >*/
  gpointer                 _gst_reserved[GST_PADDING];
};

GType   meego_gst_h264_encoder_get_type          (void);

/* virtual class function wrappers */



G_END_DECLS

#endif /* __MEEGO_GST_H264_ENCODER_H__ */
